@@ -33,7 +33,9 @@ PacketQueue::PacketQueue():
   packet_counts(nullptr),
   deleting(false),
   keep_keyframes(false),
-  warned_count(0)
+  warned_count(0),
+  has_out_of_order_packets_(false),
+  max_keyframe_interval_(0)
 {
 }
 
@@ -86,13 +88,48 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
   }
   {
     std::unique_lock<std::mutex> lck(mutex);
+    if (deleting or zm_terminate) return false;
+
+    if (!has_out_of_order_packets_ and (add_packet->packet->dts != AV_NOPTS_VALUE)) {
+      auto rit = pktQueue.rbegin();
+      // Find the previous packet for the stream, and check dts
+      while (rit != pktQueue.rend()) {
+        std::shared_ptr<ZMPacket> prev_packet = *rit;
+
+        if (prev_packet->packet->stream_index == add_packet->packet->stream_index) {
+          if (prev_packet->packet->dts > add_packet->packet->dts) {
+            Debug(1, "Have out of order packets");
+            ZM_DUMP_PACKET(prev_packet->packet, "queued_packet");
+            ZM_DUMP_PACKET(add_packet->packet, "add_packet");
+            has_out_of_order_packets_ = true;
+          }
+          break;
+        }
+        rit++;
+      }  // end while
+    }
+
+    if (!max_keyframe_interval_ and add_packet->keyframe and (video_stream_id==add_packet->packet->stream_index)) {
+      auto rit = pktQueue.rbegin();
+      int packet_count = 0;
+      while (rit != pktQueue.rend()) {
+        if ((*rit)->packet->stream_index == add_packet->packet->stream_index) {
+          if ((*rit)->keyframe) break;
+          packet_count ++;
+        }
+        ++rit;
+      }
+      Debug(1, "Have keyframe interval: %d", packet_count);
+      max_keyframe_interval_ = packet_count;
+    }
 
     pktQueue.push_back(add_packet);
-    packet_counts[add_packet->packet.stream_index] += 1;
+    packet_counts[add_packet->packet->stream_index] += 1;
     Debug(2, "packet counts for %d is %d",
-        add_packet->packet.stream_index,
-        packet_counts[add_packet->packet.stream_index]);
+        add_packet->packet->stream_index,
+        packet_counts[add_packet->packet->stream_index]);
 
+    /* Any iterators that are pointing to the end will now point to the newly pushed packet */
     for (
         auto iterators_it = iterators.begin();
         iterators_it != iterators.end();
@@ -105,7 +142,7 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
     }  // end foreach iterator
 
     if (
-        (add_packet->packet.stream_index == video_stream_id)
+        (add_packet->packet->stream_index == video_stream_id)
         and
         (max_video_packet_count > 0)
         and
@@ -115,14 +152,14 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
         warned_count++;
         Warning("You have set the max video packets in the queue to %u."
             " The queue is full. Either Analysis is not keeping up or"
-            " your camera's keyframe interval is larger than this setting."
-            , max_video_packet_count);
+            " your camera's keyframe interval %d is larger than this setting."
+            , max_video_packet_count, max_keyframe_interval_);
       }
 
       for (
           // Start at second packet because the first is always a keyframe
           auto it = ++pktQueue.begin();
-          //it != pktQueue.end() and  // can't git end because we added our packet
+          //it != pktQueue.end() and  // can't hit end because we added our packet
           (*it != add_packet) && !(deleting or zm_terminate);
           // iterator is incremented by erase
       ) {
@@ -130,10 +167,10 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
 
         ZMLockedPacket lp(zm_packet);
         if (!lp.trylock()) {
-          if (warned_count <2) {
+          if (warned_count < 2) {
             warned_count++;
             // Can't delete a locked packet, but can delete one after it.
-            Warning("Found locked packet when trying to free up video packets. This basically means that decoding is not keeping up.");
+            Warning("Found locked packet when trying to free up video packets. This means that decoding is not keeping up.");
           }
           ++it;
           continue;
@@ -152,21 +189,24 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
           }
         }  // end foreach iterator
 
+        zm_packet->decoded = true; // Have to in case analysis is waiting on it
+        zm_packet->notify_all();
+
         it = pktQueue.erase(it);
-        packet_counts[zm_packet->packet.stream_index] -= 1;
+        packet_counts[zm_packet->packet->stream_index] -= 1;
         Debug(1,
             "Deleting a packet with stream index:%d image_index:%d with keyframe:%d, video frames in queue:%d max: %d, queuesize:%zu",
-            zm_packet->packet.stream_index,
+            zm_packet->packet->stream_index,
             zm_packet->image_index,
             zm_packet->keyframe,
             packet_counts[video_stream_id],
             max_video_packet_count,
             pktQueue.size());
 
-        if (zm_packet->packet.stream_index == video_stream_id)
+        if (zm_packet->packet->stream_index == video_stream_id)
           break;
       }  // end while
-    } else {
+    } else if (warned_count > 0) {
       warned_count--;
     }  // end if not able catch up
   }  // end lock scope
@@ -179,30 +219,30 @@ bool PacketQueue::queuePacket(std::shared_ptr<ZMPacket> add_packet) {
 
 void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
   // Only do queueCleaning if we are adding a video keyframe, so that we guarantee that there is one.
-  // No good.  Have to satisfy two conditions: 
+  // No good.  Have to satisfy two conditions:
   // 1. packetqueue starts with a video keyframe
   // 2. Have minimum # of video packets
   // 3. No packets can be locked
   // 4. No iterator can point to one of the packets
   //
-  // So start at the beginning, counting video packets until the next keyframe.  
+  // So start at the beginning, counting video packets until the next keyframe.
   // Then if deleting those packets doesn't break 1 and 2, then go ahead and delete them.
   //
   // One assumption that we can make is that there will be packets in the queue. Because we call it while holding a locked packet
   if (deleting) return;
 
   if (keep_keyframes and ! (
-        add_packet->packet.stream_index == video_stream_id
+        add_packet->packet->stream_index == video_stream_id
         and
         add_packet->keyframe
         and
         (packet_counts[video_stream_id] > pre_event_video_packet_count)
-        and 
+        and
         *(pktQueue.begin()) != add_packet
         )
      ) {
     Debug(3, "stream index %d ?= video_stream_id %d, keyframe %d, keep_keyframes %d,  counts %d > pre_event_count %d at begin %d",
-        add_packet->packet.stream_index, video_stream_id, add_packet->keyframe, keep_keyframes, packet_counts[video_stream_id], pre_event_video_packet_count, 
+        add_packet->packet->stream_index, video_stream_id, add_packet->keyframe, keep_keyframes, packet_counts[video_stream_id], pre_event_video_packet_count,
         ( *(pktQueue.begin()) != add_packet )
         );
     return;
@@ -215,23 +255,19 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
     packetqueue_iterator it = pktQueue.end();
     --it;
     while (*it != add_packet) {
-      if ((*it)->packet.stream_index == video_stream_id)
+      if ((*it)->packet->stream_index == video_stream_id)
         ++tail_count;
       --it;
     }
   }
-  Debug(1, "Tail count is %d, queue size is %lu", tail_count, pktQueue.size());
+  Debug(1, "Tail count is %d, queue size is %" __PRIPTR_PREFIX "u", tail_count, pktQueue.size());
 
   if (!keep_keyframes) {
     // If not doing passthrough, we don't care about starting with a keyframe so logic is simpler
     while ((*pktQueue.begin() != add_packet) and (packet_counts[video_stream_id] > pre_event_video_packet_count + tail_count)) {
       std::shared_ptr<ZMPacket> zm_packet = *pktQueue.begin();
-      ZMLockedPacket *lp = new ZMLockedPacket(zm_packet);
-      if (!lp->trylock()) {
-        delete lp;
-        break;
-      }
-      delete lp;
+      ZMLockedPacket lp(zm_packet);
+      if (!lp.trylock()) break;
 
       if (is_there_an_iterator_pointing_to_packet(zm_packet)) {
         Warning("Found iterator at beginning of queue. Some thread isn't keeping up");
@@ -239,10 +275,10 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
       }
 
       pktQueue.pop_front();
-      packet_counts[zm_packet->packet.stream_index] -= 1;
+      packet_counts[zm_packet->packet->stream_index] -= 1;
       Debug(1,
             "Deleting a packet with stream index:%d image_index:%d with keyframe:%d, video frames in queue:%d max: %d, queuesize:%zu",
-            zm_packet->packet.stream_index,
+            zm_packet->packet->stream_index,
             zm_packet->image_index,
             zm_packet->keyframe,
             packet_counts[video_stream_id],
@@ -258,58 +294,70 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
   // First packet is special because we know it is a video keyframe and only need to check for lock
   std::shared_ptr<ZMPacket> zm_packet = *it;
   if (zm_packet == add_packet) {
+    Debug(1, "First packet in queue is the analysis packet.");
     return;
   }
 
+  int keyframe_interval_count = 1;
+  int video_packets_to_delete = 0;    // This is a count of how many packets we will delete so we know when to stop looking
+
   ZMLockedPacket *lp = new ZMLockedPacket(zm_packet);
-  if (lp->trylock()) {
-    int keyframe_interval = 1;
-    int video_packets_to_delete = 0;    // This is a count of how many packets we will delete so we know when to stop looking
-    Debug(4, "Have lock on first packet");
-    ++it;
+  if (!lp->trylock()) {
+    Debug(4, "Failed getting lock on first packet");
+    delete lp;
+    return;
+  }  // end if first packet not locked
+
+  if (is_there_an_iterator_pointing_to_packet(zm_packet)) {
+    Debug(3, "Found iterator Counted %d video packets. Which would leave %d in packetqueue tail count is %d",
+        video_packets_to_delete, packet_counts[video_stream_id]-video_packets_to_delete, tail_count);
+    delete lp;
+    return;
+  }
+
+  ++it;
+  delete lp;
+
+  // Since we have many packets in the queue, we should NOT be pointing at end so don't need to test for that
+  while (*it != add_packet) {
+    zm_packet = *it;
+    lp = new ZMLockedPacket(zm_packet);
+    if (!lp->trylock()) {
+      Debug(3, "Failed locking packet %d", zm_packet->image_index);
+      delete lp;
+      break;
+    }
     delete lp;
 
-    // Since we have many packets in the queue, we should NOT be pointing at end so don't need to test for that
-    while (*it != add_packet) {
-      zm_packet = *it;
-      lp = new ZMLockedPacket(zm_packet);
-      if (!lp->trylock()) {
-        Debug(3, "Failed locking packet %d", zm_packet->image_index);
-        delete lp;
-        break;
-      }
-      delete lp;
+    if (is_there_an_iterator_pointing_to_packet(zm_packet)) {
+      Debug(3, "Found iterator Counted %d video packets. Which would leave %d in packetqueue tail count is %d",
+          video_packets_to_delete, packet_counts[video_stream_id]-video_packets_to_delete, tail_count);
+      break;
+    }
 
-#if 0
-      // There are no threads that follow analysis thread.  So there cannot be an it pointing here
-      if (is_there_an_iterator_pointing_to_packet(zm_packet)) {
-        if (pktQueue.begin() == next_front)
-          Warning("Found iterator at beginning of queue. Some thread isn't keeping up");
-        break;
+    if (zm_packet->packet->stream_index == video_stream_id) {
+      keyframe_interval_count++;
+      if (zm_packet->keyframe) {
+        Debug(3, "Have a video keyframe so setting next front to it. Keyframe interval so far is %d", keyframe_interval_count);
+        if (keyframe_interval_count > max_keyframe_interval_)
+          max_keyframe_interval_ = keyframe_interval_count;
+        keyframe_interval_count = 1;
+        next_front = it;
       }
-#endif
-
-      if (zm_packet->packet.stream_index == video_stream_id) {
-        if (zm_packet->keyframe) {
-          Debug(1, "Have a video keyframe so setting next front to it. Keyframe interval so far is %d", keyframe_interval);
-          keyframe_interval = 1;
-          next_front = it;
-        } else {
-          keyframe_interval++;
-        }
-        ++video_packets_to_delete;
+      ++video_packets_to_delete;
+      if (packet_counts[video_stream_id] - video_packets_to_delete <= pre_event_video_packet_count + tail_count) {
         Debug(3, "Counted %d video packets. Which would leave %d in packetqueue tail count is %d",
             video_packets_to_delete, packet_counts[video_stream_id]-video_packets_to_delete, tail_count);
-        if (packet_counts[video_stream_id] - video_packets_to_delete <= pre_event_video_packet_count + tail_count) {
-          break;
-        }
+        break;
       }
-      ++it;
     } // end while
+    ++it;
   }  // end if first packet not locked
-  Debug(1, "Resulting it pointing at latest packet? %d, next front points to begin? %d",
+
+  Debug(1, "Resulting it pointing at latest packet? %d, next front points to begin? %d, Keyframe interval %d",
       ( *it == add_packet ),
-      ( next_front == pktQueue.begin() )
+      ( next_front == pktQueue.begin() ),
+      keyframe_interval_count
       );
   if (next_front != pktQueue.begin()) {
     while (pktQueue.begin() != next_front) {
@@ -321,17 +369,16 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
 
       Debug(1,
             "Deleting a packet with stream index:%d image_index:%d with keyframe:%d, video frames in queue:%d max: %d, queuesize:%zu",
-            zm_packet->packet.stream_index,
+            zm_packet->packet->stream_index,
             zm_packet->image_index,
             zm_packet->keyframe,
             packet_counts[video_stream_id],
             pre_event_video_packet_count,
             pktQueue.size());
       pktQueue.pop_front();
-      packet_counts[zm_packet->packet.stream_index] -= 1;
+      packet_counts[zm_packet->packet->stream_index] -= 1;
     }
   }  // end if have at least max_video_packet_count video packets remaining
-  // We signal on every packet because someday we may analyze sound
 
 	return;
 } // end voidPacketQueue::clearPackets(ZMPacket* zm_packet)
@@ -339,10 +386,14 @@ void PacketQueue::clearPackets(const std::shared_ptr<ZMPacket> &add_packet) {
 void PacketQueue::stop() {
   deleting = true;
   condition.notify_all();
+  for (const auto &p : pktQueue) {
+    p->notify_all();
+  }
 }
 
 void PacketQueue::clear() {
   deleting = true;
+  // Why are we notifying?
   condition.notify_all();
   if (!packet_counts) // special case, not initialised
     return;
@@ -356,13 +407,13 @@ void PacketQueue::clear() {
     lp->lock();
     Debug(1,
         "Deleting a packet with stream index:%d image_index:%d with keyframe:%d, video frames in queue:%d max: %d, queuesize:%zu",
-        packet->packet.stream_index,
+        packet->packet->stream_index,
         packet->image_index,
         packet->keyframe,
         packet_counts[video_stream_id],
         pre_event_video_packet_count,
         pktQueue.size());
-    packet_counts[packet->packet.stream_index] -= 1;
+    packet_counts[packet->packet->stream_index] -= 1;
     pktQueue.pop_front();
     delete lp;
   }
@@ -377,7 +428,7 @@ void PacketQueue::clear() {
     *iterator_it = pktQueue.begin();
   }  // end foreach iterator
 
-  if (packet_counts) delete[] packet_counts;
+  delete[] packet_counts;
   packet_counts = nullptr;
   max_stream_id = -1;
 
@@ -397,6 +448,32 @@ int PacketQueue::packet_count(int stream_id) {
   return packet_counts[stream_id];
 }  // end int PacketQueue::packet_count(int stream_id)
 
+ZMLockedPacket *PacketQueue::get_packet_no_wait(packetqueue_iterator *it) {
+  if (deleting or zm_terminate)
+    return nullptr;
+
+  Debug(4, "Locking in get_packet using it %p queue end? %d",
+      std::addressof(*it), (*it == pktQueue.end()));
+
+  // scope for lock
+  std::unique_lock<std::mutex> lck(mutex);
+  Debug(4, "Have Lock in get_packet");
+  if ((*it == pktQueue.end()) and !(deleting or zm_terminate)) {
+    Debug(2, "waiting.  Queue size %zu it == end? %d", pktQueue.size(), (*it == pktQueue.end()));
+    condition.wait(lck);
+  }
+  if ((*it == pktQueue.end()) or deleting or zm_terminate) return nullptr;
+
+  std::shared_ptr<ZMPacket> p = *(*it);
+  ZMLockedPacket *lp = new ZMLockedPacket(p);
+  if (lp->trylock()) {
+    Debug(2, "Locked packet %d, unlocking packetqueue mutex", p->image_index);
+    return lp;
+  }
+  delete lp;
+  return nullptr;
+}
+
 // Returns a packet. Packet will be locked
 ZMLockedPacket *PacketQueue::get_packet(packetqueue_iterator *it) {
   if (deleting or zm_terminate)
@@ -520,7 +597,7 @@ bool PacketQueue::increment_it(packetqueue_iterator *it, int stream_id) {
   std::unique_lock<std::mutex> lck(mutex);
   do {
     ++(*it);
-  } while ( (*it != pktQueue.end()) and ( (*(*it))->packet.stream_index != stream_id) );
+  } while ( (*it != pktQueue.end()) and ( (*(*it))->packet->stream_index != stream_id) );
 
   if ( *it != pktQueue.end() ) {
     Debug(2, "Incrementing %p, still not at end, so incrementing", it);
@@ -537,30 +614,26 @@ packetqueue_iterator *PacketQueue::get_event_start_packet_it(
 
   packetqueue_iterator *it = new packetqueue_iterator;
   iterators.push_back(it);
+  Debug(4, "Have event start iterator %p", std::addressof(*it));
 
   *it = snapshot_it;
   std::shared_ptr<ZMPacket> packet = *(*it);
-  ZM_DUMP_PACKET(packet->packet, "");
+  //ZM_DUMP_PACKET(packet->packet, "");
   // Step one count back pre_event_count frames as the minimum
   // Do not assume that snapshot_it is video
   // snapshot it might already point to the beginning
-  if (pre_event_count) {
-    while ((*it) != pktQueue.begin()) {
-      packet = *(*it);
-      Debug(1, "Previous packet pre_event_count %d stream_index %d keyframe %d score %d",
-          pre_event_count, packet->packet.stream_index, packet->keyframe, packet->score);
-      ZM_DUMP_PACKET(packet->packet, "");
-      if (packet->packet.stream_index == video_stream_id) {
-        pre_event_count --;
-        if (!pre_event_count)
-          break;
-      }
-      (*it)--;
-    }
+  while (pre_event_count and ((*it) != pktQueue.begin())) {
+    /*
+    Debug(1, "Previous packet pre_event_count %d stream_index %d keyframe %d score %d",
+        pre_event_count, packet->packet->stream_index, packet->keyframe, packet->score);
+    ZM_DUMP_PACKET(packet->packet, "");
+    */
+    if (packet->packet->stream_index == video_stream_id)
+      pre_event_count --;
+    (*it)--;
     packet = *(*it);
   }
-  // it either points to beginning or we have seen pre_event_count video packets.
-  
+
   if (pre_event_count) {
     if (packet->image_index < (int)pre_event_count) {
       // probably just starting up
@@ -568,21 +641,22 @@ packetqueue_iterator *PacketQueue::get_event_start_packet_it(
     } else {
       Warning("Hit end of packetqueue before satisfying pre_event_count. Needed %d more video frames", pre_event_count);
     }
-    ZM_DUMP_PACKET(packet->packet, "");
+    //ZM_DUMP_PACKET(packet->packet, "");
     return it;
   } else if (!keep_keyframes) {
     // Are encoding, so don't care about keyframes
+    // We could be pointing to a non-video frame though.  Do we care?
     return it;
   }
 
   while ((*it) != pktQueue.begin()) {
-    packet = *(*it);
-    ZM_DUMP_PACKET(packet->packet, "No keyframe");
-    if ((packet->packet.stream_index == video_stream_id) and packet->keyframe)
+    //ZM_DUMP_PACKET(packet->packet, "No keyframe");
+    if ((packet->packet->stream_index == video_stream_id) and packet->keyframe)
       return it; // Success
     --(*it);
+    packet = *(*it);
   }
-  if (!(*(*it))->keyframe) {
+  if (!packet->keyframe) {
     Warning("Hit beginning of packetqueue and packet is not a keyframe. index is %d", packet->image_index);
   }
   return it;
@@ -627,18 +701,19 @@ packetqueue_iterator * PacketQueue::get_video_it(bool wait) {
       return nullptr;
     }
     Debug(1, "Packet keyframe %d for stream %d, so returning the it to it",
-        zm_packet->keyframe, zm_packet->packet.stream_index);
-    if (zm_packet->keyframe and ( zm_packet->packet.stream_index == video_stream_id )) {
+        zm_packet->keyframe, zm_packet->packet->stream_index);
+    if (zm_packet->keyframe and ( zm_packet->packet->stream_index == video_stream_id )) {
       Debug(1, "Found a keyframe for stream %d, so returning the it to it", video_stream_id);
       return it;
     }
     ++(*it);
   }
-  Debug(1, "DIdn't Found a keyframe for stream %d, so returning the it to it", video_stream_id);
+  Debug(1, "Didn't find a keyframe for stream %d, so returning the it to it", video_stream_id);
   return it;
 }  // get video_it
 
 void PacketQueue::free_it(packetqueue_iterator *it) {
+  std::unique_lock<std::mutex> lck(mutex);
   for (
       std::list<packetqueue_iterator *>::iterator iterators_it = iterators.begin();
       iterators_it != iterators.end();
@@ -659,6 +734,7 @@ bool PacketQueue::is_there_an_iterator_pointing_to_packet(const std::shared_ptr<
       ) {
     packetqueue_iterator *iterator_it = *iterators_it;
     if (*iterator_it == pktQueue.end()) {
+      Debug(4, "Checking iterator %p == end", std::addressof(*iterator_it));
       continue;
     }
     Debug(4, "Checking iterator %p == packet ? %d", std::addressof(*iterator_it), ( *(*iterator_it) == zm_packet ));
